from typing import Optional, Dict, Any, Union
from pydantic.fields import Field
from uuid import UUID

from .core import SystemCommandBaseModel


class RunIntegrationAction(SystemCommandBaseModel):
    integration_id: Union[UUID, str] = Field(
        None,
        title="Integration ID",
        description="The unique ID of the Integration.",
    )
    action_id: str = Field(
        None,
        title="Action Identifier",
        description="A string identifier of the action of the related integration.",
    )
    config_overrides: Optional[Dict[str, Any]] = Field(
        None,
        title="Configuration",
        description="A dictionary with the configuration used to execute the action",
    )



from gundi_core.events.core import SystemEventBaseModel


class SystemCommandBaseModel(SystemEventBaseModel):
    # Marker class for system commands.
    # Technically, a command message is equal to an event message but expressing an intent.
    pass



from typing import Optional
from pydantic import BaseModel, Field
from gundi_core.schemas.v2 import DispatchedObservation, UpdatedObservation, CustomDispatcherLog
from .core import SystemEventBaseModel


class DispatcherErrorDetails(BaseModel):
    error: str = Field(
        "",
        title="Error",
        description="A human-readable string describing the error.",
    )
    error_traceback: Optional[str] = Field(
        "",
        title="Error Traceback",
        description="A string with the traceback of the error.",
    )
    server_response_status: Optional[int] = Field(
        None,
        title="Server Response Status",
        description="The status code of the server response.",
    )
    server_response_body: Optional[str] = Field(
        "",
        title="Server Response",
        description="The response from the server as text.",
    )


class DeliveryErrorDetails(DispatcherErrorDetails):
    observation: Optional[DispatchedObservation] = Field(
        None,
        title="Observation",
        description="The observation that caused the error.",
    )


class UpdateErrorDetails(DispatcherErrorDetails):
    observation: Optional[UpdatedObservation] = Field(
        None,
        title="Observation",
        description="The observation that caused the error.",
    )


# Events emmited by dispatchers

class ObservationDelivered(SystemEventBaseModel):
    payload: DispatchedObservation


class ObservationDeliveryFailed(SystemEventBaseModel):
    schema_version: str = Field("v2", const=True)
    payload: DeliveryErrorDetails


class ObservationUpdated(SystemEventBaseModel):
    payload: UpdatedObservation


class ObservationUpdateFailed(SystemEventBaseModel):
    schema_version: str = Field("v2", const=True)
    payload: UpdateErrorDetails


class DispatcherCustomLog(SystemEventBaseModel):
    payload: CustomDispatcherLog






import pydantic
from gundi_core.schemas.v2 import (
    IntegrationSummary,
    IntegrationActionConfiguration,
    IntegrationConfigChanges,
    ActionConfigChanges,
    IntegrationDeletionDetails,
    ActionConfigDeletionDetails,
)
from .core import SystemEventBaseModel

# Events published by the portal on config changes


class IntegrationCreated(SystemEventBaseModel):
    payload: IntegrationSummary

class IntegrationUpdated(SystemEventBaseModel):
    payload: IntegrationConfigChanges

class IntegrationDeleted(SystemEventBaseModel):
    payload: IntegrationDeletionDetails


class ActionConfigCreated(SystemEventBaseModel):
    payload: IntegrationActionConfiguration

class ActionConfigUpdated(SystemEventBaseModel):
    payload: ActionConfigChanges

class ActionConfigDeleted(SystemEventBaseModel):
    payload: ActionConfigDeletionDetails
